#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

@author: banixcyan

@contact: banixcyan@gmail.com

@time: 2017/9/30 14:57

@desc: log工具
    通过名字获得唯一的log
    console_logger直接打印在控制台中
"""
import logging
import logging.handlers

try:
    import cloghandler
except ImportError:
    cloghandler = None

logger_set = set()

default_message_fmt = '[%(asctime)s.%(msecs)01d][%(process)d][%(filename)s:%(lineno)s][%(levelname)s][%(message)s]'
default_date_fmt = '%Y-%m-%d %H:%M:%S'
default_log_level = logging.DEBUG

CONSOLE_LOG_NAME = 'std_in_out_err'
LOG_SIZE = 1024 * 1024 * 100  # 100M


def init_log(log_path, name='__default', filemode='a', message_fmt=default_message_fmt, date_fmt=default_date_fmt,
             level=logging.DEBUG):
    if name in logger_set:
        return logging.getLogger(name)

    try:
        if cloghandler:
            handler = cloghandler.ConcurrentRotatingFileHandler(log_path, filemode, 1024 * 1024 * 100, 10)
        else:
            handler = logging.handlers.RotatingFileHandler(log_path, filemode, 1024 * 1024 * 100, 10)
    except IOError as e:
        console_logger.error(repr(e))
        return exit(1)

    formatter = logging.Formatter(message_fmt, date_fmt)
    handler.setFormatter(formatter)

    logger = logging.getLogger(name)
    logger.setLevel(level)
    logger.addHandler(handler)
    logger_set.add(name)

    return logger


def get_log(name='__default'):
    if name in logger_set:
        return logging.getLogger(name)
    else:
        return None


def get_console_logger(level=logging.DEBUG):
    logger_name = 'std_in_out_err'
    if logger_name in logger_set:
        return logging.getLogger(logger_name)

    handler = logging.StreamHandler()
    formatter = logging.Formatter(default_message_fmt, default_date_fmt)
    handler.setFormatter(formatter)

    log = logging.getLogger(logger_name)
    log.setLevel(level)
    log.addHandler(handler)

    logger_set.add(logger_name)
    return log


console_logger = get_console_logger()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

@author: banixcyan

@contact: banixcyan@gmail.com

@time: 2017/12/11 16:26

@desc: config

    读取配置文件

"""
import json

try:
    import toml
except ImportError:
    toml = None

config_dict = {}


def init_conf(filename, name='__default'):
    with open(filename) as fp:
        data = fp.read()
        decode_data = __try_decode(data, json.loads)

        if decode_data:
            config_dict[name] = decode_data
            return decode_data

        if toml:
            decode_data = __try_decode(data, toml.loads)
            if decode_data:
                config_dict[name] = decode_data
                return decode_data
    return None


def get_conf(name='__default'):
    return config_dict[name]


def __try_decode(data, call):
    try:
        return call(data)
    except ValueError:
        return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

@author: banixcyan

@license: (C) Copyright 2013-2017, Tencent Corporation Limited.

@contact: banixcyan@gmail.com

@time: 2017/12/11 16:25

@desc: Py工具集封装 集成config, db, log

"""
from pykvf.log import console_logger as clog
from pykvf.config import init_conf, get_conf
from pykvf.db import init_db, get_db
from pykvf.log import init_log, get_log, get_console_logger


def init(config_filename, name='__default'):
    c = init_conf(config_filename, name)

    for l in c['log']:
        init_log(l['filename'], name=l.get('name', '__default'), level=l.get('level', 10))

    for d in c['mysql']:
        init_db(d['host'], d['user'], d['password'], d['port'], d['database'], alias=d.get('name', None), max_conn=d.get('max_conn', 10))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from distutils.core import setup
from setuptools import find_packages

setup(
    name='pykvf',
    version='0.1.12',
    description='null',
    author='banixc',
    author_email='banixc@qq.com',
    url=' ',
    py_modules=['pykvf'],
    packages=find_packages(),
    install_requires=[
        'pymysql',
        'toml'
    ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

@author: banixcyan

@contact: banixcyan@gmail.com

@time: 2017/9/30 14:57

@desc: DB

    连接池 sql拼装 sql预格式化 参数检查

"""
import sys

import pymysql
from pymysql import InternalError, OperationalError

try:
    import queue
except Exception:
    import Queue as queue

db_dict = {}


class DB:
    def __init__(self, host, user, password, port, database, charset, max_conn):
        self.db_config = {
            'host': host,
            'user': user,
            'password': password,
            'port': port,
            'database': database,
            'charset': charset
        }
        self.init_error_message = None
        self.conn_pool = queue.Queue(maxsize=max_conn)
        for _ in range(max_conn):
            self.conn_pool.put(self.init_conn())
        self.callback_function = {}

    def init_conn(self):
        return pymysql.connect(**self.db_config)

    def ping(self, conn):
        try:
            cursor = conn.cursor()
            cursor.execute('select 1;')
            cursor.fetchall()
            conn.commit()
            cursor.close()
            return conn
        except (OperationalError, InternalError):
            return self.init_conn()

    def get_conn(self):
        return self.ping(self.conn_pool.get())

    def put_conn(self, conn):
        conn.rollback()
        self.conn_pool.put(conn)

    def session(self):
        return Session(self)

    def exe(self, sql, args=None):
        session = self.session()
        affect_rows = session.exe(sql, args)
        session.commit()
        return affect_rows

    def query(self, sql, args=None):
        return self.session().query(sql, args)

    def select(self, field, table, where=None, other=None, args=None):
        return self.query(select(field, table, where, other), args)

    def update(self, table, field, where=None, other=None, args=None):
        return self.exe(update(table, field, where, other), args)

    def insert(self, table, field, args):
        return self.exe(insert(table, field), args)

    def delete(self, table, where=None, other=None, args=None):
        return self.exe(delete(table, where, other), args)

    def replace(self, table, field, args):
        return self.exe(replace(table, field), args)

    def insert_on_duplicate_key(self, table, field, args):
        return self.exe(insert_on_duplicate_key(table, field), args)

    def callback(self, operation_type, func):
        self.callback_function[operation_type] = func


class Session:
    def __init__(self, db_instance):
        self.db = db_instance
        self.conn = db_instance.get_conn()
        self.cursor = self.conn.cursor(cursor=pymysql.cursors.DictCursor)

    def __del__(self):
        self.db.put_conn(self.conn)

    def __exe(self, sql, args=None):
        callback = self.db.callback_function.get(sys._getframe().f_code.co_name, None)
        if callback is not None:
            callback(sql, args)
        return self.cursor.execute(sql, args)

    def __fetchall(self):
        return self.cursor.fetchall()

    def query(self, sql, args=None):
        callback = self.db.callback_function.get(sys._getframe().f_code.co_name, None)
        if callback is not None:
            callback(sql, args)
        self.__exe(sql, args)
        return self.__fetchall()

    def exe(self, sql, args=None):
        callback = self.db.callback_function.get(sys._getframe().f_code.co_name, None)
        if callback is not None:
            callback(sql, args)
        return self.__exe(sql, args)

    def select(self, field, table, where=None, other=None, args=None):
        return self.query(select(field, table, where, other), args)

    def insert(self, table, field, args):
        return self.exe(insert(table, field), args)

    def update(self, table, field, where=None, other=None, args=None):
        return self.exe(update(table, field, where, other), args)

    def delete(self, table, where=None, other=None, args=None):
        return self.exe(delete(table, where, other), args)

    def replace(self, table, field, args):
        return self.exe(replace(table, field), args)

    def insert_on_duplicate_key(self, table, field, args):
        return self.exe(insert_on_duplicate_key(table, field), args)

    def commit(self):
        self.conn.commit()

    def rollback(self):
        self.conn.rollback()

    def get_insert_id(self):
        return self.conn.insert_id()


def select(field, table, where, other):
    return 'SELECT ' + ', '.join(field) + ' FROM ' + table + where_and_other(where, other)


def insert(table, field):
    return 'INSERT INTO ' + table + ' ( ' + ', '.join(field) + ') VALUES (' + ('%s, ' * len(field))[:-2] + ')'


def update(table, field, where, other):
    return 'UPDATE ' + table + ' SET ' + ', '.join(field) + where_and_other(where, other)


def delete(table, where, other):
    return 'DELETE FROM ' + table + where_and_other(where, other)


def replace(table, field):
    return 'REPLACE INTO ' + table + ' ( ' + ', '.join(field) + ') VALUES (' + ('%s, ' * len(field))[:-2] + ')'


def insert_on_duplicate_key(table, field):
    return insert(table, field) + " ON DUPLICATE KEY UPDATE " + ", ".join(
        ["`{0}`=VALUES(`{0}`)".format(f) for f in field])


def where_and_other(where, other):
    sql = ''
    if where is not None and len(where) > 0:
        sql += ' WHERE (' + ') AND ('.join(where) + ')'
    if other is not None and other != '':
        sql += ' ' + other
    return sql


def init_db(host='localhost', user='root', password='root', port=3306, database='test', charset='utf8', alias=None,
            max_conn=10):
    if alias is None:
        alias = database
    db_dict[alias] = DB(host, user, password, port, database, charset, max_conn)
    return db_dict[alias]


def get_db(name=None):
    if name is None:
        name = list(db_dict.keys())[0]
    return db_dict[name]
